//
//  Generic24BitImage.m
//  PostureRecognizer
//
//  Created by Sebastian Hans on Thu Sep 02 2004.
//  Copyright (c) 2004 Sebastian Hans. All rights reserved.
//

#import "Generic24BitImage.h"


@implementation Generic24BitImage

- (Generic24BitImage*) fromNSImage: (NSImage*) source
{
	NSImageRep *rep;
	unsigned char *orig_data;
	int i, bpp;

	rep = [source bestRepresentationForDevice: nil];
	width = [rep pixelsWide];
	height = [rep pixelsHigh];
	if (data)
		free(data);
	if (!(data = malloc(3 * width * height)))
		return nil;
	orig_data = [rep bitmapData];
	bpp = [rep bitsPerPixel];
	switch (bpp)
	{
	case 24:
		for (i = 0; i < 3 * width * height; i++)
			data[i] = orig_data[i];
		break;
	default:
		printf("Bitmap with unknown depth encountered: %d", bpp);
	}
	return self;
}

- (Generic24BitImage*) fromIplImage: (IplImage*) source
{
	int i;
	
	width = source->width;
	height = source->height;
	if (data)
		free(data);
	if (!(data = malloc(3 * width * height)))
		return nil;
	for (i = 0; i < 3 * width * height; i++)
		data[i] = source->imageData[i];
	return self;
}

- (NSImage*) toNSImage
{
	NSSize size;
	unsigned char *imagedata;
	NSBitmapImageRep *rep;
	NSImage *image;
	int i;

	size.width = width;
	size.height = height;
	rep = [NSBitmapImageRep alloc];
	[rep initWithBitmapDataPlanes: NULL pixelsWide: width pixelsHigh: height
			bitsPerSample: 8 samplesPerPixel: 3 hasAlpha: NO isPlanar: NO
			colorSpaceName: NSCalibratedRGBColorSpace bytesPerRow: 0 bitsPerPixel: 0];
	image = [[NSImage alloc] initWithSize: size];
	[image addRepresentation: rep];
	imagedata = [rep bitmapData];
	for (i = 0; i < 3 * width * height; i++)
		imagedata[i] = data[i];
	return image;
}

- (IplImage*) toIplImage
{
	CvSize size;
	IplImage *image;
	int i;

	size.width = width;
	size.height = height;
	image = cvCreateImage(size, IPL_DEPTH_8U, 3);
	for (i = 0; i < 3 * width * height; i++)
		image->imageData[i] = data[i];
	return image;
}

- (void) dealloc
{
	if (data)
		free(data);
}

@end
