//
//  Generic8BitImage.m
//  PostureRecognizerTest 
//
//  Created by Sebastian Hans on Sun Aug 29 2004.
//  Copyright (c) 2004 Sebastian Hans. All rights reserved.
//

#import "Generic8BitImage.h"


@implementation Generic8BitImage

- (Generic8BitImage*) fromNSImage: (NSImage*) source
{
	NSImageRep *rep;
	unsigned char *orig_data;
	int i, bpp;

	rep = [source bestRepresentationForDevice: nil];
	width = [rep pixelsWide];
	height = [rep pixelsHigh];
	if (data)
		free(data);
	if (!(data = malloc(width * height)))
		return nil;
	orig_data = [rep bitmapData];
	bpp = [rep bitsPerPixel];
	switch (bpp)
	{
	case 1:
		for (i = 0; i < width * height; i++)
			data[i] = orig_data[i] ? 255 : 0;
		break;
	case 8:
		for (i = 0; i < width * height; i++)
			data[i] = orig_data[i];
		break;
	case 24:
		for (i = 0; i < width * height; i++)
			data[i] = orig_data[3 * i] > orig_data[3 * i + 1] ?
					orig_data[3 * i] > orig_data[3 * i + 2] ?
						orig_data[3 * i] : orig_data[3 * i + 2] :
					orig_data[3 * i + 1] > orig_data[ 3 * i + 2] ?
						orig_data[3 * i + 1] : orig_data[3 * i + 2];
		break;
	default:
		printf("Bitmap with unknown depth encountered: %d", bpp);
	}
	return self;
}

- (Generic8BitImage*) fromIplImage: (IplImage*) source
{
	int i;
	
	width = source->width;
	height = source->height;
	if (data)
		free(data);
	if (!(data = malloc(width * height)))
		return nil;
	for (i = 0; i < width * height; i++)
		data[i] = source->imageData[i];
	return self;
}

- (NSImage*) toNSImage
{
	NSSize size;
	unsigned char *imagedata;
	NSBitmapImageRep *rep;
	NSImage *image;
	int i;

	size.width = width;
	size.height = height;
	rep = [NSBitmapImageRep alloc];
	[rep initWithBitmapDataPlanes: NULL pixelsWide: width pixelsHigh: height
			bitsPerSample: 8 samplesPerPixel: 3 hasAlpha: NO isPlanar: NO
			colorSpaceName: NSCalibratedRGBColorSpace bytesPerRow: 0 bitsPerPixel: 0];
	image = [[NSImage alloc] initWithSize: size];
	[image addRepresentation: rep];
	imagedata = [rep bitmapData];
	for (i = 0; i < width * height; i++)
		imagedata[3 * i] = imagedata[3 * i + 1] = imagedata[3 * i + 2] = data[i];
	return image;
}

- (IplImage*) toIplImage
{
	CvSize size;
	IplImage *image;
	int i;

	size.width = width;
	size.height = height;
	image = cvCreateImage(size, IPL_DEPTH_8U, 1);
	for (i = 0; i < width * height; i++)
		image->imageData[i] = data[i];
	return image;
}

- (void) dealloc
{
	if (data)
		free(data);
}

@end
