//
//  PostureRecognizerController.m
//  PostureRecognizerTest
//
//  Created by Sebastian Hans on Fri Aug 27 2004.
//  Copyright (c) 2004 Sebastian Hans. All rights reserved.
//

#import "PostureRecognizerController.h"
#import "Generic8BitImage.h"
#import "Generic24BitImage.h"
#import "WhiteOnGreenSegmentor.h"
#import <stdio.h>

@implementation PostureRecognizerController

- (void) awakeFromNib
{
	[super awakeFromNib];
	segmentor = [[WhiteOnGreenSegmentor alloc] init];
	model_list = cp_create_model_list([numPointsField intValue]);
	match_list = NULL;
	current_model = NULL;
	next_model_id = 1;
	current_difference_stepper_value = 0;
	current_image = current_segmented_image = 0;
	[modelComboBox setHasVerticalScroller: YES];
}

- (void) displayImage
{
	if ([segmentedImageCheckbox state] == NSOnState)
		[imageView setImage: current_segmented_image];
	else
		[imageView setImage: current_image];
}

- (void) displayModel
{
	Generic8BitImage *img8;

	if (!current_model)
		[modelView setImage: nil];
	else if ([segmentedModelCheckbox state] == NSOnState)
	{
		img8 = [((Generic8BitImage*) [Generic8BitImage alloc])
				fromIplImage: current_model->image];
		[modelView setImage: [img8 toNSImage]];
		[img8 release];
	}
	else
		[modelView setImage: (NSImage*) current_model->user_data];
}

- (Generic8BitImage*) segment: (NSImage*) image
{
	Generic24BitImage *image24;
	Generic8BitImage *segmented;

	image24 = [((Generic24BitImage*) [Generic24BitImage alloc]) fromNSImage: image];
	segmented = [segmentor segment: image24];
	[image24 release];
	return segmented;
}

- (IBAction) processImage: (id) sender
{
	Generic8BitImage *segmented;

	if (current_image)
		[current_image release];
	current_image = [[sender image] copy];
	segmented = [self segment: current_image];
	if (current_segmented_image)
		[current_segmented_image release];
	current_segmented_image = [segmented toNSImage];
	[segmented release];
	[self displayImage];
	[self rematch: sender];
}

- (IBAction) rematch: (id) sender
{
	Generic8BitImage *generic_img;
	IplImage *ipl_img;
	cp_match_t *best_match, *match;

	if (match_list)
	{
		[differenceStepper setEnabled: NO];
		cp_destroy_match_list(match_list);
		match_list = NULL;
	}
	if (!model_list->length)
	{
		puts("No models in list. Add models by dragging them into the left window.\n");
		return;
	}

	if (!current_segmented_image)
	{
		puts("No image to match.");
		return;
	}

	generic_img = [((Generic8BitImage*) [Generic8BitImage alloc])
			fromNSImage: current_segmented_image];
	ipl_img = [generic_img toIplImage];
	[generic_img release];
	match_list = cp_match(ipl_img, model_list, &best_match);
	cvReleaseImage(&ipl_img);

	if (!match_list)
	{
		puts("Got no match list. 'Sup?");
		return;
	}
	if (!match_list->length)
	{
		puts("Empty match list.");
		return;
	}

	cp_sort_match_list(match_list);

	for (match = match_list->first; match; match = match->next)
	{
		if (match->model->name)
			printf("Model %ld (%s) matches with %g, rel. orient: %g, rel. size: %g\n",
					match->model->id, match->model->name, match->difference,
					match->relative_orientation, match->relative_size);
		else
			printf("Model %ld matches with %g, rel. orient: %g, rel. size: %g\n",
					match->model->id, match->difference, match->relative_orientation,
					match->relative_size);
		if (!best_match || match->difference < best_match->difference)
			best_match = match;
	}
	[self setCurrentModel: best_match->model];
	[differenceStepper setIntValue: current_difference_stepper_value];
	[differenceStepper setEnabled: YES];
}

- (void) setCurrentModel: (cp_model_t*) model
{
	Generic8BitImage *generic_img;
	NSString *name;

	current_model = model;
	[self displayModel];
	if (model)
	{
		name = [self cString2NSString: model->name];
		[modelComboBox setStringValue: name];
		[modelComboBox selectItemWithObjectValue: name];
	}
	else
		[modelComboBox setStringValue: @""];
	[self updateDifference];
}

- (void) updateDifference
{
	cp_match_t *match = NULL;
	char diff_buf[12];

	if (match_list)
		match = cp_find_match_for_model(match_list, current_model);
	if (match)
	{
		snprintf(diff_buf, 12, "%6.3f", match->difference);
		diff_buf[11] = '\0';
		[differenceField setTitleWithMnemonic: [self cString2NSString: diff_buf]];
	}
	else
		[differenceField setTitleWithMnemonic: [self cString2NSString: "-"]];
}

- (NSString*) cString2NSString: (char*) s
{
	return [[NSString alloc] initWithCStringNoCopy: s length: strlen(s) freeWhenDone: NO];
}

- (void) addModel: (NSImage*) image as: (const char*) name
{
	Generic8BitImage *img;
	char *_name;

	img = [self segment: image];
	if (!(_name = strdup(name)))
	{
		puts("No memory for the model name?");
		[img release];
		return;
	}
	printf("Adding model '%s'.\n", _name);
	cp_append_model_with_user_data(model_list, [img toIplImage], next_model_id,
			_name, [image copy]);
	[img release];
	[modelComboBox addItemWithObjectValue: [self cString2NSString: _name]];
	if (current_image)
		[self rematch: self];
	else
		[self setCurrentModel: cp_find_model_name(model_list, _name)];
	next_model_id++;
}

- (IBAction) addModel: (id) sender
{
	char name_buf[256];

	snprintf(name_buf, 256, "Model %ld", next_model_id);
	name_buf[255] = '\0';
	[self addModel: [sender image] as: name_buf];
}

- (IBAction) clearImage: (id) sender
{
	if (match_list)
	{
		[differenceStepper setEnabled: NO];
		cp_destroy_match_list(match_list);
		match_list = NULL;
	}
	if (current_segmented_image)
		[current_segmented_image release];
	current_image = current_segmented_image = nil;
	[self displayImage];
	[self updateDifference];
}

- (IBAction) enterModel: (id) sender
{
	char *name;
	cp_model_t *model;
	int i;

	if (!model_list->length)
		return;

	name = [[modelComboBox stringValue] cString];
	if (!strcmp("", name))
	{
		[self setCurrentModel: current_model];
		return;
	}
	if (model = cp_find_model_name(model_list, name))
	{
		[self setCurrentModel: model];
		return;
	}
	i = [modelComboBox indexOfItemWithObjectValue:
			[self cString2NSString: current_model->name]];
	[modelComboBox removeItemAtIndex: i];
	free(current_model->name);
	current_model->name = strdup(name);
	[modelComboBox insertItemWithObjectValue: [self cString2NSString: current_model->name]
			atIndex: i];
}

- (IBAction) removeModel: (id) sender
{
	cp_model_t *next;

	if (!model_list->length)
		return;
	if (current_model->next)
		next = current_model->next;
	else if (current_model->prev)
		next = current_model->prev;
	else
		next = NULL;
	cp_remove_model(model_list, current_model);
	[modelComboBox removeItemWithObjectValue: [self cString2NSString: current_model->name]];
	cp_destroy_model(current_model, 1, 0);
	[self setCurrentModel: next];
	if (current_image)
		[self rematch: self];
}

- (IBAction) useModelAsImage: (id) sender
{
	if ([modelView image])
	{
		[imageView setImage: (NSImage*) current_model->user_data];
		[self processImage: imageView];
	}
}

- (IBAction) useImageAsModel: (id) sender
{
	if (current_image)
	{
		[modelView setImage: current_image];
		[self addModel: modelView];
	}
}

- (IBAction) stepDifference: (id) sender
{
	cp_match_t *match;
	
	if(!(match = cp_find_match_for_model(match_list, current_model)))
		return;

	if ([sender intValue] == (current_difference_stepper_value + 1) % 3)
	{
		/* step up */
		if (match == match_list->last)
			match = match_list->first;
		else
			match = match->next;
	}
	else
	{
		/* step down */
		if (match == match_list->first)
			match = match_list->last;
		else
			match = match->prev;
	}
	current_difference_stepper_value = [sender intValue];
	[self setCurrentModel: match->model];
}

- (IBAction) setNumPoints: (id) sender
{
	int num_points;
	
	num_points = [sender intValue];
	if (num_points < 2)
	{
		[sender setIntValue: model_list->num_points];
		return;
	}
	if (!cp_set_num_points_for_model_list(model_list, num_points))
	{
		puts("Something went wrong changing num_points.");
		puts("I suggest restarting the program since it is now severely fucked up.");
	}
	[self rematch: self];
}

- (IBAction) modelDisplayChange: (id) sender
{
	[self displayModel];
}

- (IBAction) imageDisplayChange: (id) sender
{
	[self displayImage];
}

- (void) dealloc
{
	[segmentor release];
	if (current_image)
		[current_image release];
	if (current_segmented_image)
		[current_segmented_image release];
	cp_destroy_model_list(model_list, 1, 0);
	if (match_list)
		cp_destroy_match_list(match_list);
	[super dealloc];
}

- (BOOL) addModelFile: (NSString*) filename
{
	NSImage *image;
	NSArray* components;
	
	if (!(image = [[NSImage alloc] initWithContentsOfFile: filename]))
		return NO;
	components = [filename componentsSeparatedByString: @"/"];
	[self addModel: image as: [((NSString*) [components lastObject]) cString]];
	return YES;
}

- (BOOL) application: (NSApplication*) theApplication openFile: (NSString*) filename
{
	return [self addModelFile: filename];
}

- (void) addModelFiles: (NSArray*) filenames
{
	NSEnumerator *enumerator;
	NSString *filename;

	enumerator = [filenames objectEnumerator];
	while (filename = [enumerator nextObject])
		[self addModelFile: filename];
}

- (IBAction) openModels: (id) sender
{
    NSArray *fileTypes = [NSArray arrayWithObjects: @"png", @"jpeg", @"jpg", nil];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];

    [oPanel setAllowsMultipleSelection: YES];
    if ([oPanel runModalForDirectory:NSHomeDirectory() file: nil types: fileTypes] ==
			NSOKButton)
        [self addModelFiles: [oPanel filenames]];
}

@end
