//
//  WhiteOnGreenSegmentor.m
//  PostureRecognizer
//
//  Created by Sebastian Hans on Thu Sep 02 2004.
//  Copyright (c) 2004 Sebastian Hans. All rights reserved.
//

#import "WhiteOnGreenSegmentor.h"
#import "Set.h"


@implementation WhiteOnGreenSegmentor


- (id) init
{
	return self;
}

- (IplImage*) getLargestConnectedComponent: (IplImage*) image
{
	int width, height, i, j;
	unsigned char *data;
	Set **sets, **regions, *largest_region = nil;
	IplImage *result;
	long next_set = 0, largest_size = 0;
	
	width = cvGetSize(image).width;
	height = cvGetSize(image).height;
	if (!(sets = malloc(sizeof(*sets) * width * height)))
		return NULL;
	if (!(regions = malloc(sizeof(*regions) * width * height)))
		return NULL;
	if (!(result = cvCreateImage(cvGetSize(image), IPL_DEPTH_8U, 1)))
	{
		free(regions);
		return NULL;
	}
	cvZero(result);

	data = image->imageData;

#define SET_UNITED(i, j, i2, j2)	do \
									{ \
										regions[width * (j) + (i)] = \
											[regions[width * (j) + (i)] \
													uniteWith: \
													regions[width * (j2) + (i2)]]; \
									} \
									while (0)

	for (j = 0; j < height; j++)
		for (i = 0; i < width; i++)
		{
			if (!data[width * j + i])
			{
				regions[width * j + i] = nil;
				continue;
			}
			regions[width * j + i] = sets[next_set++] = [[Set alloc] init];
			if (i && data[width * j + i - 1])
				SET_UNITED(i, j, i - 1, j);
			if (i && j && data[width * (j - 1) + i - 1])
				SET_UNITED(i, j, i - 1, j - 1);
			if (j && data[width * (j - 1) + i])
				SET_UNITED(i, j, i, j - 1);
			if (j && i != width - 1 && data[width * (j - 1) + i + 1])
				SET_UNITED(i, j, i + 1, j - 1);
		}
	
	for (i = 0; i < next_set; i++)
		if (!largest_region || [sets[i] getSize] > largest_size)
		{
			largest_region = sets[i];
			largest_size = [sets[i] getSize];
		}

	if (largest_region)
	{
		data = result->imageData;
		for (j = 0; j < height; j++)
			for (i = 0; i < width; i++)
			{
				Set *region;

				region = [regions[width * j + i] find];
				data[width * j + i] = (region == largest_region) ? 255 : 0;
			}
	}

	for (i = 0; i < next_set; i++)
		[sets[i] release];
	free(sets);
	free(regions);
	return result;
}

- (Generic8BitImage*) segment: (Generic24BitImage*) image
{
	IplImage *ipl_image, *red_plane, *thresholded, *thresholded2, *largest_component;
	IplImage *largest_dilated, *largest_and_thresholded;
	IplConvKernel *struc_el;
	Generic8BitImage *segmented_image;

	ipl_image = [image toIplImage];
	red_plane = cvCreateImage(cvGetSize(ipl_image), IPL_DEPTH_8U, 1);
	cvCvtPixToPlane(ipl_image, red_plane, NULL, NULL, NULL);
	cvReleaseImage(&ipl_image);
	thresholded = cvCreateImage(cvGetSize(red_plane), IPL_DEPTH_8U, 1);
	cvThreshold(red_plane, thresholded, 190, 255, CV_THRESH_BINARY);
	cvReleaseImage(&red_plane);
	thresholded2 = cvCloneImage(thresholded);
	struc_el = cvCreateStructuringElementEx(7, 7, 3, 3, CV_SHAPE_ELLIPSE, NULL);
	cvErode(thresholded2, thresholded2, struc_el, 1);
	cvReleaseStructuringElement(&struc_el);
	largest_component = [self getLargestConnectedComponent: thresholded2];
	cvReleaseImage(&thresholded2);

/**/
	cvReleaseImage(&thresholded);
	segmented_image = [((Generic8BitImage*) [Generic8BitImage alloc])
			fromIplImage: largest_component];
	cvReleaseImage(&largest_component);
	return segmented_image;
/**/

	struc_el = cvCreateStructuringElementEx(63, 63, 31, 31, CV_SHAPE_ELLIPSE, NULL);
	cvDilate(largest_component, largest_component, struc_el, 1);
	cvReleaseStructuringElement(&struc_el);
	largest_and_thresholded = cvCreateImage(cvGetSize(largest_component), IPL_DEPTH_8U, 1);
	cvAnd(largest_component, thresholded, largest_and_thresholded, NULL);
	cvReleaseImage(&largest_component);
	cvReleaseImage(&thresholded);
	segmented_image = [((Generic8BitImage*) [Generic8BitImage alloc])
			fromIplImage: largest_and_thresholded];
	cvReleaseImage(&largest_and_thresholded);
	return segmented_image;
}

@end
